@@ -3,6 +3,7 @@
 from typing import Dict, Tuple
 import pandas as pd
 import numpy as np
+import json
 
 
 def build_dgl_dataset(nodes: Dict[str, pd.DataFrame],
@@ -55,6 +56,8 @@ def build_dgl_dataset(nodes: Dict[str, pd.DataFrame],
     dgl_graph = dgl.heterograph(graph_data)
 
     def emb_to_tensor(df: pd.DataFrame, col_name: str):
+        if type(df[col_name].iloc[0]) == str:
+            df[col_name] = df[col_name].map(lambda x: json.loads(x))
         np_array = np.stack(df[col_name].tolist())
         if len(np_array.shape) == 1:
             np_array = np.expand_dims(np_array, axis=1)
@@ -138,20 +141,20 @@ def build_dgl_dataset(nodes: Dict[str, pd.DataFrame],
 
     # Add node features to the Claim nodes
     if 'claim' in nodes.keys() and 'claim' in dgl_graph.ntypes:
+        claim_embs = emb_to_tensor(nodes['claim'], 'embedding')
         if 'reviewer_emb' in nodes['claim'].columns:
+            claim_embs = emb_to_tensor(nodes['claim'], 'embedding')
             rev_embs = emb_to_tensor(nodes['claim'], 'reviewer_emb')
-            dgl_graph.nodes['claim'].data['feat'] = rev_embs
+            tensors = (claim_embs, rev_embs)
+            dgl_graph.nodes['claim'].data['feat'] = torch.cat(tensors, dim=1)
         else:
-            num_claims = dgl_graph.num_nodes('claim')
-            dgl_graph.nodes['claim'].data['feat'] = torch.ones(num_claims, 1)
+            dgl_graph.nodes['claim'].data['feat'] = claim_embs
 
     # Add labels
     def numericalise_labels(label: str) -> int:
         numericalise = dict(misinformation=0, factual=1, other=2)
         return numericalise[label]
-    claim_labels = (nodes['claim'][['predicted_verdict']]
-                    .applymap(numericalise_labels)
-                    .rename(columns=dict(predicted_verdict='label')))
+    claim_labels = nodes['claim'][['label']].applymap(numericalise_labels)
     discusses = relations[('tweet', 'discusses', 'claim')]
     tweet_labels = (nodes['tweet'].merge(discusses.merge(claim_labels,
                                                          left_on='tgt',
@@ -165,4 +168,22 @@ def build_dgl_dataset(nodes: Dict[str, pd.DataFrame],
     dgl_graph.nodes['claim'].data['label'] = claim_label_tensor
     dgl_graph.nodes['tweet'].data['label'] = tweet_label_tensor
 
+    # Add masks
+    mask_names = ['train_mask', 'val_mask', 'test_mask']
+    claim_masks = nodes['claim'][mask_names].copy()
+    merged = (nodes['tweet'].merge(discusses.merge(claim_masks,
+                                                   left_on='tgt',
+                                                   right_index=True)
+                                            .drop_duplicates('src'),
+                                   left_index=True,
+                                   right_on='src',
+                                   how='left'))
+    for col_name in mask_names:
+        claim_tensor = torch.from_numpy(nodes['claim'][col_name].astype(int)
+                                                                .to_numpy())
+        tweet_tensor = torch.from_numpy(merged[col_name].astype(int)
+                                                        .to_numpy())
+        dgl_graph.nodes['claim'].data[col_name] = claim_tensor
+        dgl_graph.nodes['tweet'].data[col_name] = tweet_tensor
+
     return dgl_graph
